#!/bin/bash
#
declare -a FAILED_PARAMS_LIST
declare -a KERNEL_PARAMS_LIST
declare -A KERNEL_PARAMS_VALUE_REQUIRED

KERNEL_PARAMS_LIST=(fs.file-max vm.max_map_count vm.min_free_kbytes net.core.somaxconn vm.swappiness net.ipv4.ip_local_port_range fs.inotify.max_user_watches)
KERNEL_PARAMS_VALUE_REQUIRED["fs.file-max"]=65536
KERNEL_PARAMS_VALUE_REQUIRED["vm.max_map_count"]=262144
KERNEL_PARAMS_VALUE_REQUIRED["vm.min_free_kbytes"]=1024000
KERNEL_PARAMS_VALUE_REQUIRED["net.core.somaxconn"]=2048
KERNEL_PARAMS_VALUE_REQUIRED["vm.swappiness"]=0
KERNEL_PARAMS_VALUE_REQUIRED["net.ipv4.ip_local_port_range"]="65300 1024"
KERNEL_PARAMS_VALUE_REQUIRED["net.ipv4.ip_local_port_range_reduce"]=$((65300-1024))
KERNEL_PARAMS_VALUE_REQUIRED["fs.inotify.max_user_watches"]=1048576



for _kp in ${KERNEL_PARAMS_LIST[@]}; do
    if sysctl -q $_kp &> /dev/null ; then
        if [ "$_kp" != "net.ipv4.ip_local_port_range" ]; then
            system_value=$(sysctl -q $_kp | awk -F "=" '{gsub("\\s","");print $NF}')
            if [[ $(awk 'BEGIN{print('"$system_value"' >= '"${KERNEL_PARAMS_VALUE_REQUIRED[$_kp]}"')?0:1}') -ne 0 ]]; then
                FAILED_PARAMS_LIST[${#FAILED_PARAMS_LIST[@]}]=$_kp
            fi
        else
            system_value=$(sysctl -q $_kp | awk -F "[ |=]" '{gsub("\\t"," ");print $5-$4}')
            if [[ $(awk 'BEGIN{print('"$system_value"' >= '"${KERNEL_PARAMS_VALUE_REQUIRED["${_kp}_reduce"]}"')?0:1}') -ne 0 ]]; then
                FAILED_PARAMS_LIST[${#FAILED_PARAMS_LIST[@]}]=$_kp
            fi
        fi
    else
        FAILED_PARAMS_LIST[${#FAILED_PARAMS_LIST[@]}]=$_kp
    fi
done

if [ -z "${FAILED_PARAMS_LIST}" ]; then
    echo "{\"checkValue\": \"true\",\"checkStatus\": \"OK\", \"msg\": \"All kernel paramaters cheked, no issue found.\"}"
else
    echo "{\"checkValue\": \"true\",\"checkStatus\": \"fail\", \"msg\": \"$(for((i=0;i<${#FAILED_PARAMS_LIST[@]};i++)); do  printf "${FAILED_PARAMS_LIST[$i]} check FAILED, required: ${KERNEL_PARAMS_VALUE_REQUIRED[${FAILED_PARAMS_LIST[$i]}]};" ;done)\"}"
fi